@@ -50,21 +50,21 @@ my $debug = 0;
 my $host_arch = get_host_arch();
 
 sub version {
-    printf _g("Debian %s version %s.\n"), $Dpkg::PROGNAME, $Dpkg::PROGVERSION;
+    printf g_("Debian %s version %s.\n"), $Dpkg::PROGNAME, $Dpkg::PROGVERSION;
 
-    printf _g('
+    printf g_('
 This is free software; see the GNU General Public License version 2 or
 later for copying conditions. There is NO warranty.
 ');
 }
 
 sub usage {
-    printf _g(
+    printf g_(
 'Usage: %s [<option>...]')
-    . "\n\n" . _g(
+    . "\n\n" . g_(
 'Options:
   -p<package>              generate symbols file for package.
-  -P<package-build-dir>    temporary build dir instead of debian/tmp.
+  -P<package-build-dir>    temporary build directory instead of debian/tmp.
   -e<library>              explicitly list libraries to scan.
   -v<version>              version of the packages (defaults to
                            version extracted from debian/changelog).
@@ -96,7 +96,7 @@ while (@ARGV) {
     if (m/^-p/p) {
 	$oppackage = ${^POSTMATCH};
 	my $err = pkg_name_is_illegal($oppackage);
-	error(_g("illegal package name '%s': %s"), $oppackage, $err) if $err;
+	error(g_("illegal package name '%s': %s"), $oppackage, $err) if $err;
     } elsif (m/^-c(\d)?$/) {
 	$compare = $1 // 1;
     } elsif (m/^-q$/) {
@@ -112,7 +112,7 @@ while (@ARGV) {
 	} else {
 	    my @to_add = glob($file);
 	    push @files, @to_add;
-	    warning(_g("pattern '%s' did not match any file"), $file)
+	    warning(g_("pattern '%s' did not match any file"), $file)
 		unless scalar(@to_add);
 	}
     } elsif (m/^-P(.+)$/) {
@@ -137,10 +137,12 @@ while (@ARGV) {
 	version();
 	exit(0);
     } else {
-	usageerr(_g("unknown option \`%s'"), $_);
+	usageerr(g_("unknown option '%s'"), $_);
     }
 }
 
+report_options(debug_level => $debug);
+
 umask 0022; # ensure sane default permissions for created files
 
 if (exists $ENV{DPKG_GENSYMBOLS_CHECK_LEVEL}) {
@@ -155,9 +157,9 @@ if (not defined($oppackage)) {
     my $control = Dpkg::Control::Info->new();
     my @packages = map { $_->{'Package'} } $control->get_packages();
     if (@packages == 0) {
-	error(_g('no package stanza found in control info'));
+	error(g_('no package stanza found in control info'));
     } elsif (@packages > 1) {
-	error(_g('must specify package since control info has many (%s)'),
+	error(g_('must specify package since control info has many (%s)'),
 	      "@packages");
     }
     $oppackage = $packages[0];
@@ -171,7 +173,7 @@ foreach my $file ($input, $output, "debian/$oppackage.symbols.$host_arch",
     'debian/symbols')
 {
     if (defined $file and -e $file) {
-	print "Using references symbols from $file\n" if $debug;
+	debug(1, "Using references symbols from $file");
 	$symfile->load($file);
 	$ref_symfile->load($file) if $compare || ! $quiet;
 	last;
@@ -193,7 +195,7 @@ if (not scalar @files) {
             next PATH if -l $updir;
         }
 	opendir(my $libdir_dh, "$libdir")
-	    or syserr(_g("can't read directory %s: %s"), $libdir, $!);
+	    or syserr(g_("can't read directory %s: %s"), $libdir, $!);
 	push @files, grep {
 	    /(\.so\.|\.so$)/ && -f &&
 	    Dpkg::Shlibs::Objdump::is_elf($_);
@@ -205,28 +207,28 @@ if (not scalar @files) {
 # Merge symbol information
 my $od = Dpkg::Shlibs::Objdump->new();
 foreach my $file (@files) {
-    print "Scanning $file for symbol information\n" if $debug;
+    debug(1, "Scanning $file for symbol information");
     my $objid = $od->analyze($file);
     unless (defined($objid) && $objid) {
-	warning(_g("Dpkg::Shlibs::Objdump couldn't parse %s\n"), $file);
+	warning(g_("Dpkg::Shlibs::Objdump couldn't parse %s\n"), $file);
 	next;
     }
     my $object = $od->get_object($objid);
     if ($object->{SONAME}) { # Objects without soname are of no interest
-	print "Merging symbols from $file as $object->{SONAME}\n" if $debug;
+	debug(1, "Merging symbols from $file as $object->{SONAME}");
 	if (not $symfile->has_object($object->{SONAME})) {
 	    $symfile->create_object($object->{SONAME}, "$oppackage #MINVER#");
 	}
 	$symfile->merge_symbols($object, $sourceversion);
     } else {
-	print "File $file doesn't have a soname. Ignoring.\n" if $debug;
+	debug(1, "File $file doesn't have a soname. Ignoring.");
     }
 }
 $symfile->clear_except(keys %{$od->{objects}});
 
 # Write out symbols files
 if ($stdout) {
-    $output = _g('<standard output>');
+    $output = g_('<standard output>');
     $symfile->output(\*STDOUT, package => $oppackage,
                      template_mode => $template_mode,
                      with_pattern_matches => $verbose_output,
@@ -239,13 +241,13 @@ if ($stdout) {
 	}
     }
     if (defined($output)) {
-	print "Storing symbols in $output.\n" if $debug;
+	debug(1, "Storing symbols in $output.");
 	$symfile->save($output, package => $oppackage,
 	               template_mode => $template_mode,
 	               with_pattern_matches => $verbose_output,
 	               with_deprecated => $verbose_output);
     } else {
-	print "No symbol information to store.\n" if $debug;
+	debug(1, 'No symbol information to store.');
     }
 }
 
@@ -254,23 +256,23 @@ my $exitcode = 0;
 if ($compare || ! $quiet) {
     # Compare
     if (my @libs = $symfile->get_new_libs($ref_symfile)) {
-	warning(_g('new libraries appeared in the symbols file: %s'), "@libs")
+	warning(g_('new libraries appeared in the symbols file: %s'), "@libs")
 	    unless $quiet;
 	$exitcode = 4 if ($compare >= 4);
     }
     if (my @libs = $symfile->get_lost_libs($ref_symfile)) {
-	warning(_g('some libraries disappeared in the symbols file: %s'), "@libs")
+	warning(g_('some libraries disappeared in the symbols file: %s'), "@libs")
 	    unless $quiet;
 	$exitcode = 3 if ($compare >= 3);
     }
     if ($symfile->get_new_symbols($ref_symfile)) {
-	warning(_g('some new symbols appeared in the symbols file: %s'),
-		_g('see diff output below')) unless $quiet;
+	warning(g_('some new symbols appeared in the symbols file: %s'),
+		g_('see diff output below')) unless $quiet;
 	$exitcode = 2 if ($compare >= 2);
     }
     if ($symfile->get_lost_symbols($ref_symfile)) {
-	warning(_g('some symbols or patterns disappeared in the symbols file: %s'),
-	        _g('see diff output below')) unless $quiet;
+	warning(g_('some symbols or patterns disappeared in the symbols file: %s'),
+	        g_('see diff output below')) unless $quiet;
 	$exitcode = 1 if ($compare >= 1);
     }
 }
@@ -278,30 +280,40 @@ if ($compare || ! $quiet) {
 unless ($quiet) {
     require File::Temp;
     require Digest::MD5;
+
+    my $file_label;
+
     # Compare template symbols files before and after
     my $before = File::Temp->new(TEMPLATE=>'dpkg-gensymbolsXXXXXX');
     my $after = File::Temp->new(TEMPLATE=>'dpkg-gensymbolsXXXXXX');
+    if ($ref_symfile->{file}) {
+        $file_label = $ref_symfile->{file};
+    } else {
+        $file_label = 'new_symbol_file';
+    }
     $ref_symfile->output($before, package => $oppackage, template_mode => 1);
     $symfile->output($after, package => $oppackage, template_mode => 1);
-    seek($before, 0, 0); seek($after, 0, 0);
+
+    seek $before, 0, 0;
+    seek $after, 0, 0;
     my ($md5_before, $md5_after) = (Digest::MD5->new(), Digest::MD5->new());
     $md5_before->addfile($before);
     $md5_after->addfile($after);
+
     # Output diffs between symbols files if any
     if ($md5_before->hexdigest() ne $md5_after->hexdigest()) {
 	if (not defined($output)) {
-	    warning(_g('the generated symbols file is empty'));
+	    warning(g_('the generated symbols file is empty'));
 	} elsif (defined($ref_symfile->{file})) {
-	    warning(_g("%s doesn't match completely %s"),
+	    warning(g_("%s doesn't match completely %s"),
 		    $output, $ref_symfile->{file});
 	} else {
-	    warning(_g('no debian/symbols file used as basis for generating %s'),
+	    warning(g_('no debian/symbols file used as basis for generating %s'),
 		    $output);
 	}
 	my ($a, $b) = ($before->filename, $after->filename);
-	my $diff_label = sprintf('%s (%s_%s_%s)',
-	($ref_symfile->{file}) ? $ref_symfile->{file} : 'new_symbol_file',
-	$oppackage, $sourceversion, $host_arch);
+	my $diff_label = sprintf('%s (%s_%s_%s)', $file_label, $oppackage,
+	                         $sourceversion, $host_arch);
 	system('diff', '-u', '-L', $diff_label, $a, $b) if find_command('diff');
     }
 }
