@@ -2,7 +2,7 @@
  * dpkg - main program for package management
  * filesdb.h - management of database of files installed on system
  *
- * Copyright © 1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright © 1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
  * Copyright © 2008-2014 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
@@ -113,6 +113,13 @@ struct fileinlist {
   struct filenamenode *namenode;
 };
 
+/**
+ * Queue of filenamenode entries.
+ */
+struct filenamenode_queue {
+  struct fileinlist *head, **tail;
+};
+
 /**
  * When we deal with an ‘overridden’ file, every package except the
  * overriding one is considered to contain the other file instead. Both
@@ -145,6 +152,7 @@ struct pkginfo *filepackages_iter_next(struct filepackages_iterator *iter);
 void filepackages_iter_free(struct filepackages_iterator *iter);
 
 void filesdbinit(void);
+void files_db_reset(void);
 
 struct fileiterator;
 struct fileiterator *files_db_iter_new(void);
@@ -155,10 +163,15 @@ void ensure_package_clientdata(struct pkginfo *pkg);
 
 void ensure_diversions(void);
 
+enum statdb_parse_flags {
+	STATDB_PARSE_NORMAL = 0,
+	STATDB_PARSE_LAX = 1,
+};
+
 uid_t statdb_parse_uid(const char *str);
 gid_t statdb_parse_gid(const char *str);
 mode_t statdb_parse_mode(const char *str);
-void ensure_statoverrides(void);
+void ensure_statoverrides(enum statdb_parse_flags flags);
 
 #define LISTFILE           "list"
 #define HASHFILE           "md5sums"
@@ -170,9 +183,9 @@ void note_must_reread_files_inpackage(struct pkginfo *pkg);
 struct filenamenode *findnamenode(const char *filename, enum fnnflags flags);
 void parse_filehash(struct pkginfo *pkg, struct pkgbin *pkgbin);
 void write_filelist_except(struct pkginfo *pkg, struct pkgbin *pkgbin,
-                           struct fileinlist *list, enum fnnflags mask);
+                           struct fileinlist *list, enum filenamenode_flags mask);
 void write_filehash_except(struct pkginfo *pkg, struct pkgbin *pkgbin,
-                           struct fileinlist *list, enum fnnflags mask);
+                           struct fileinlist *list, enum filenamenode_flags mask);
 
 struct reversefilelistiter { struct fileinlist *todo; };
 
